# frozen_string_literal: true

module Hermes
  # HousingRequest serializer
  class HousingRequestSerializer < Hermes::ApplicationSerializer
    cache key: proc { "#{object.cache_key}" }, expires_in: 1.day
    type 'housing_requests'

    attributes :backoffice_status,
               :is_active,
               :als_id,
               :request_status,
               :attributed_housing_district,
               :attributed_housing_district_postal_code,
               :old_request_status,
               :request_status_updated_at,
               :old_entity_status,
               :entity_status_updated_at,
               :entity_status,
               :dalo,
               :asked_mobility,
               :valid_mobility,
               :decider,
               :submitted_to_cal_nb,
               :sent_to_cal_nb,
               :rejected_in_cal_nb,
               :rejected_after_cal_nb,
               :accepted_in_cal_nb,
               :mobility,
               :seeked_types,
               :entity_help_required,
               :contract_type,
               :registration_number,
               :applicant_ids,
               :entity_validated,
               :priority_points_indexes,
               :has_priority,
               :nud,
               :nb_of_requests_made,
               :reference_tax_revenues,
               :seeked_districts,
               :category_type,
               :single_parent,
               :id_counter,
               :id_unique,
               :id_unique_associate,
               :internal_ref,
               :forcing_duplicate,
               :electronical_renewal,
               :ec_law_ressources,
               :is_long_delay_anormal,
               :date_calculator_indicator,
               :with_car_park,
               :widen_request,
               :housing_type_list,
               :expulsion_judgement_date,
               :other_reason,
               :work_contract_type,
               :seeked_district_postal_codes,
               :seeked_departments,
               :seeked_localisation_list,
               :dependents,
               :monthly_revenues,
               :dependents_monthly_incomes,
               :dependents_monthly_charges,
               :typology_range,
               :entity_validated,
               :family_size,
               :coapplicants,
               :lessor_name,
               :lessor_siren,
               :social_housing_entry_date,
               :social_housing_name,
               :test_created_at,
               :sne_number,
               :employee_id,
               :employee_start_date,
               :entity_access_level,
               :employee_unit,
               :tab_state,
               :motivation1,
               :motivation2,
               :motivation3,
               :seeking_first_floor,
               :seeking_elevator,
               :seeked_max_rent,
               :seeked_epci_first_quartils,
               :seeked_quartil_belongings,
               :sne_creation_date,
               :precisions,
               :has_multiple_attributions_under_conditions,
               :seeked_housing_categories,
               :radiation_date,
               :radiation_motive,
               :min_surface,
               :main_applicant_id,
               :main_applicant,
               :requester_firstname,
               :requester_lastname,
               :requester_fullname,
               :requester_email,
               :requester_birth_date,
               :entity_id,
               :coapplicant_ids,
               :coapplicants,
               :applications_nb,
               :last_application_date,
               :housing_application_ids,
               :active_application_id,
               :form_status,
               :profile_epci_score,
               :score_with_position,
               :applicated_position,
               :eligible_position,
               :effort_rate,
               :entity_id,
               :documents_required,
               :documents_uploaded,
               :unborn_children,
               :is_house_owner,
               :workplace_insee_code,
               :unique_numbers,
               :is_untensed_area,
               :housing_channel_chosen_keys,
               :housing_channel_pending_keys,
               :housing_channel_keys,
               :is_sne_update_available,
               :last_search_on_sne,
               :housing_offer_ids,
               :last_search_on_sne,
               :viewed_by_colaborator_offre_logement,
               :last_xml_validation_report,
               :last_send_has_error,
               :last_error_responses,
               :last_entity_code
               # :housing_applications,
               # :last_application_date,

    attribute :dependents, key_transformation: :unaltered
    attribute :disability_details, key_transformation: :unaltered
    attribute :profession_details, key_transformation: :unaltered
    attribute :actual_housing_details, key_transformation: :unaltered
    attribute :postal_details, key_transformation: :unaltered
    attribute :owned_housing_details, key_transformation: :unaltered

    attribute :eligible_offers_nb_detailed, if: :detailed_eligibility_count?

    belongs_to :housing_request_folder, serializer: HousingRequestFolderSerializer do
      object.housing_request_folder
    end

    def housing_offer_ids
      object.housing_offer_ids&.map(&:to_s) || []
    end

    def unique_numbers
      res = []
      object.nuds&.each do |nud|
        res.push(nud.unique_number)
      end
      res
    end

    def form_status
      object.form_status
    end

    def request_status
      if (object.form_status == "complete")
        if object.request_status == "INCOMPLETE" || !object.request_status
          "EN ATTENTE DE LA CNI"
        else
          object.request_status
        end
      else
        "INCOMPLETE"
      end
    end

    def detailed_eligibility_count?
      instance_options[:detailed_eligibility] == true
    end

    def eligible_offers_nb_detailed
      return "uneligible" unless object.main_applicant&.eligibility_query.present?

      eligibility_json = object.main_applicant.eligibility_query.json_to_h
      eligible_offers = Hermes::HousingOffer.where(eligibility_json)
      {
        non_commercialized: eligible_offers.only(:commercialized)
                              .where(commercialized: {:$ne => true})
                              .count,
        commercialized_publicated: eligible_offers.only(
          :commercialized,
          :is_mad
        ).where(
          commercialized: true,
          is_mad: true
        ).count,
        commercialized_non_publicated: eligible_offers.only(
          :commercialized,
          :is_mad
        ).where(
          commercialized: true,
          is_mad: {:$ne => true}
        ).count
      }
    end

    def score
      return unless instance_options[:scores]

      @score ||= instance_options[:scores].select do |score|
        score.applicant_id == object.main_applicant_id
      end.first
    end

    def handle_score_value(value)
      value.nil? ? 'uneligible' : value
    end

    def profile_epci_score
      handle_score_value score&.profile_epci_score
    end

    def score_with_position
      handle_score_value score&.score_with_position
    end

    def applicated_position
      handle_score_value score&.applicated_position
    end

    def eligible_position
      handle_score_value score&.eligible_position
    end

    def effort_rate
      handle_score_value score&.effort_rate
    end

    def main_applicant_id
      object.main_applicant_id&.to_s
    end

    def main_applicant
      return nil unless object.main_applicant

      Hermes::ApplicantSerializer.new(
        object.main_applicant,
        scope: scope,
        root: false
      )
    end

    def entity_id
      object.entity&.id.to_s
    end

    def coapplicant_ids
      object.applicant_ids&.map(&:to_s)
    end

    def coapplicants
      object.coapplicants
    end

    def housing_application_ids
      object.housing_applications&.map {|a| a.id.to_s} || []
    end

    def housing_applications
      object.housing_applications.map do |application|
        Hermes::HousingApplicationSerializer.new(
          application,
          scope: scope,
          root: false
        )
      end
    end
  end
end
